#include "boggle.h"
FILE *fp;

/* 
 * open and read dictionary file in dict structure.  the dict
 * structure is constructed using a recusive invocation of
 * dict_read.  */

dict *dict_open(char *f) {
  int i;
  dict *dp=0;
  int count=0; 

  if ((fp=fopen(f, "r"))==0) {
    fprintf(stderr, "Can't open dictonary file: %s\n", f); return(0);
  }

  /* read words from dictionary */
  while(!feof(fp)) {
    dict_read(&dp);
    if (ferror(fp)) {
      fprintf(stderr, "can't read dictionary file.\n"); return(0);
    }
    if (count++%10000==0) {printf("%d words read.\n", count-1);}
  }

  if (fclose(fp)) {
    fprintf(stderr, "Can't close dictonary file: %s\n", f); return(0);
  }
  return(dp);
}

/* 
 * read one word from the dictionary file and add it to the dict
 * structure.  For every char of the word, a recusive invocation is
 * done.  dpp points to the pointer to the block corresponding to the
 * next char on the input stream. (except for the first invocation,
 * this is the .next field of the element corresponding to the last
 * char read). */

int dict_read(dict **dpp) {
  int i;  
  char ch;

  /* read next char */
  if (fread(&ch, 1, 1, fp)==0) return(1); 
  if (ch==10) return(1);
  if (ch < 'a' || ch > 'z') {
    fprintf(stderr, "Illegal character in dictonary file: '%c'. Skipping word.\n", ch);
    while (ch!=10) {if (fread(&ch, 1, 1, fp)==0) return(1); }
    return (0);
  }

  /* if no block exists here already, create one */
  if (*dpp==0) {
    (*dpp)=malloc(26 * sizeof(dict));
    for (i=0; i<26; i++) {
      (*dpp)[i].attrib=0;
      (*dpp)[i].next=0;
    }
  }

  /* The magic recusion...  Process the next char. If we're at the end
   * of a complete word, set the leaf bit. */

  if (dict_read(&(*dpp)[ch-'a'].next) && (!feof(fp))) (*dpp)[ch-'a'].attrib=1;

  return (0); 
}

int dict_lookup(dict **dpp, char c) {
  int temp;
  temp=(*dpp)[c-'a'].attrib;
  (*dpp)=(*dpp)[c-'a'].next;
  return (temp);
}
